library(Rcpp)
library(foreach)
library(doMC)
registerDoMC()

registerPlugin("QuantLib", 
               Rcpp:::Rcpp.plugin.maker(include.before='#include <ql/quantlib.hpp>',
                                        libs="-lQuantLib"))

cppFunction("
    using namespace QuantLib;
    double Heston_PDF(double x0, double v0, double r, double q, 
                      double kappa, double theta, double sigma, double rho, 
                      double x, double v, double t) {
      return HestonProcess(
        Handle<YieldTermStructure>(boost::shared_ptr<YieldTermStructure>(
          new FlatForward(r, NullCalendar(), Handle<Quote>(
            boost::shared_ptr<Quote>(new SimpleQuote(r))), ActualActual()))),
        Handle<YieldTermStructure>(boost::shared_ptr<YieldTermStructure>(
          new FlatForward(q, NullCalendar(), Handle<Quote>(
            boost::shared_ptr<Quote>(new SimpleQuote(q))), ActualActual()))),
        Handle<Quote>(boost::shared_ptr<Quote>(
          new SimpleQuote(std::exp(x0)))), 
        v0, kappa, theta, sigma, rho).pdf(x, v, t);
    }",
  plugins=c("QuantLib")
)

x <- seq(3,6,0.05)
v <- seq(1e-2, 1.0, 0.025)

pdf <- function(x, v, sigma, rho) {
  w <- array(0,dim=c(length(x), length(v)))
  
  for (i in 1:length(x))
    w[i, ] <- unlist(
      foreach(j=1:length(v)) %dopar% {
       Heston_PDF(log(100), 0.4, 0.05, 0.025, 1.0, 0.4, sigma, rho, x[i], v[j], 1.0)
      })
  w
}


w1 <- pdf(x,v, 0.8, -0.75)
w2 <- pdf(x,v, 0.8, 0.75)
w3 <- pdf(x,v, 0.4, -0.75)
w4 <- pdf(x,v, 0.4, 0.0)

# see http://wiki.cbr.washington.edu/qerm/index.php/R/Contour_Plots 

plot.new()
par(new = "TRUE",plt = c(0.1,0.4,0.60,0.95),las = 1,cex.axis = 1)
filled.contour3(x,v,w1, color=terrain.colors, ylim=c(0.01,0.8),zlim = c(min(w3),max(w3)))
par(xpd = NA)
text(x=3.5,y=0.7,"(a)")

par(new = "TRUE", plt = c(0.5,0.8,0.60,0.95), las = 1,cex.axis = 1)
filled.contour3(x,v,w2, color=terrain.colors, ylim=c(0.01,0.8), zlim = c(min(w3),max(w3)))
par(xpd = NA)
text(x=3.5,y=0.7,"(b)")

par(new = "TRUE", plt = c(0.1,0.4,0.15,0.5), las = 1,cex.axis = 1)
filled.contour3(x,v,w3, color=terrain.colors, ylim=c(0.01,0.8), zlim = c(min(w3),max(w3)))
par(xpd = NA)
text(x=3.5,y=0.7,"(c)")

par(new = "TRUE", plt = c(0.5,0.8,0.15,0.5), las = 1,cex.axis = 1)
filled.contour3(x,v,w4, color=terrain.colors, ylim=c(0.01,0.8), zlim = c(min(w3),max(w3)))
par(xpd = NA)
text(x=3.5,y=0.7,"(d)")

par(new = "TRUE", plt = c(0.85,0.9,0.25,0.85),las = 1,cex.axis = 1)

filled.legend(x,v,w,color=terrain.colors,xlab="", ylab="",    zlim = c(min(w3),max(w3)))
